searchData={"items":[{"type":"module","title":"CEM","doc":"The primary interface for defining a CEM optimization problem and executing\nthe search.\n\nTODO:\n- summarize the functional flow of CEM optimization, with supporting diagram","ref":"CEM.html"},{"type":"module","title":"Define a problem - CEM","doc":"A `CEM` problem is a struct that holds seven (7) required functions. In many\nsituations, it will make sense to define some or all of these functions in a\nmodule and then create the problem struct using `CEM.new/1`.\n\nFor example, here's a module that defines a simple one-dimensional\noptimization problem:\n\n    defmodule MyProblem do\n      alias CEM.Helpers\n      alias CEM.Random\n      alias CEM.Stats\n\n      def init(_opts), do: %{mean: 0, std: 100}\n\n      def draw(%{mean: mean, std: std}), do: Random.normal(mean, std)\n\n      def score(x), do: :math.exp(-x * x)\n\n      def update(sample) do\n        {mean, std} = Stats.sample_mean_and_std(sample)\n        %{mean: mean, std: std}\n      end\n\n      def smooth(params, params_prev, f_interp) do\n        %{\n          mean: Helpers.interpolate(params.mean, params_prev.mean, f_interp),\n          std: Helpers.interpolate(params.std, params_prev.std, f_interp)\n        }\n      end\n\n      def terminate?([entry | _], _opts), do: entry.params.std < 0.001\n    end\n\nNow build the struct:\n\n    problem =\n      CEM.new(\n        init: &MyProblem.init/1,\n        draw: &MyProblem.draw/1,\n        score: &MyProblem.score/1,\n        update: &MyProblem.update/1,\n        smooth: &MyProblem.smooth/3,\n        terminate?: &MyProblem.terminate?/2\n      )","ref":"CEM.html#module-define-a-problem"},{"type":"module","title":"Search - CEM","doc":"Once the problem struct is defined, execute the search with `CEM.search/2`:\n\n    CEM.search(problem, opts)\n\nwhere `opts` is a keyword list of options (see the documentation for\n`CEM.search/2`). After the options are validated, `opts` is turned into a\nmap, and any functions passed to `CEM.new/1` that take `opts` as an argument\nexpect a map.","ref":"CEM.html#module-search"},{"type":"function","title":"CEM.new/1","doc":"Create a new `CEM.Problem` struct from a keyword list of required functions.\n\nThe required functions are:\n\n* `:init` (function of arity 1) - Required. Function to initialize the parameters of the probability distribution used to\n  generate candidate solutions.\n\n* `:draw` (function of arity 1) - Required. Function to draw a random instance from the probability distribution with the given\n  parameters.\n\n* `:score` (function of arity 1) - Required. Function to score a candidate solution.\n\n* `:update` (function of arity 1) - Required. Function to update the parameters of the probability distribution using a\n  sample of candidate solutions.\n\n* `:smooth` (function of arity 3) - Required. Function to smooth the parameters of the probability distribution by linearly\n  interpolating between the most recent sample-based update and the parameters\n  from the previous optimization iteration.\n\n* `:terminate?` (function of arity 2) - Function to decide if the search should be terminated based on the log so far.\n  If not provided, the search will end when the `n_step_max` is reached (see docs\n  for `CEM.search/2`).","ref":"CEM.html#new/1"},{"type":"function","title":"CEM.replace/3","doc":"Replace a required function in the `CEM.Problem` struct.","ref":"CEM.html#replace/3"},{"type":"function","title":"CEM.search/2","doc":"Search for the optimal solution instance.\n\nGiven a `CEM` struct and optional parameters, return the optimal solution\ninstance and its score, the corresponding parameters of the probabiity\ndistribution, and a log of optimization progress.","ref":"CEM.html#search/2"},{"type":"function","title":"Options - CEM.search/2","doc":"* `:n_sample` (`t:non_neg_integer/0`) - The size of the sample generated before selecting the elite set. The default value is `100`.\n\n* `:f_elite` - The fraction between 0 and 1 of the sample size used to select the elite set. The default value is `0.1`.\n\n* `:f_interp` - The fraction between 0 and 1 used to interpolate between the current and previous parameters. The default value is `0.1`.\n\n* `:mode` - The optimization mode, either `:min` (minimization) or `:max` (maximization). The default value is `:max`.\n\n* `:n_step_max` (`t:non_neg_integer/0`) - The maximum number of CEM steps at which the search is terminated.\n  This is the default termination condition when `terminate?` is not\n  provided by the user in `CEM.new/1`, and it is a fail-safe to prevent\n  infinite recursion. The default value is `100`.\n\n* `:other_opts` (`t:keyword/0`) - User-specified options. _Warning: these options are not validated._ The default value is `[]`.","ref":"CEM.html#search/2-options"},{"type":"type","title":"CEM.opts/0","doc":"Map of options used internally.","ref":"CEM.html#t:opts/0"},{"type":"type","title":"CEM.params/0","doc":"Parameters of the the probability distribution used to generate candidate\nsolutions.","ref":"CEM.html#t:params/0"},{"type":"type","title":"CEM.step/0","doc":"Iteration count for the CEM search.","ref":"CEM.html#t:step/0"},{"type":"module","title":"CEM.Helpers","doc":"Assorted helper functions.","ref":"CEM.Helpers.html"},{"type":"function","title":"CEM.Helpers.interpolate/3","doc":"Linearly interpolate between number `x` and `y` with parameter `f` is the\ninterpolation fraction from 0 to 1.\n\nAs `f` varies from 0 to 1, the result varies from `x` to `y`.","ref":"CEM.Helpers.html#interpolate/3"},{"type":"module","title":"CEM.Log","doc":"Module that defines the log of optimization progress.","ref":"CEM.Log.html"},{"type":"function","title":"CEM.Log.new/0","doc":"Create a new instance of `Log` as an empty list.","ref":"CEM.Log.html#new/0"},{"type":"function","title":"CEM.Log.update/2","doc":"Update the log by prepending a new entry with the provided step,\nparameters, and score.","ref":"CEM.Log.html#update/2"},{"type":"type","title":"CEM.Log.entry/0","doc":"","ref":"CEM.Log.html#t:entry/0"},{"type":"type","title":"CEM.Log.t/0","doc":"","ref":"CEM.Log.html#t:t/0"},{"type":"module","title":"CEM.MapMatrix","doc":"A matrix data structure based on nested maps with a very limited set of\noperations.\n\nFor certain problems, it's more performant to use `MapMatrix` operations\nthan the equivalent opertations on `Nx` tensors.","ref":"CEM.MapMatrix.html"},{"type":"function","title":"CEM.MapMatrix.from_list/1","doc":"Given a list of lists of values, return a `MapMatrix`.\n\nThis is useful when converting an `Nx` tensor to a `MapMatrix`.","ref":"CEM.MapMatrix.html#from_list/1"},{"type":"function","title":"CEM.MapMatrix.gather/2","doc":"Collect values at the given index pairs `[{i, j},...]` into a list.\n\nThis is similar to `Nx.gather/2`.","ref":"CEM.MapMatrix.html#gather/2"},{"type":"function","title":"CEM.MapMatrix.get/2","doc":"Get the value at the given index pair `{i, j}`.","ref":"CEM.MapMatrix.html#get/2"},{"type":"function","title":"CEM.MapMatrix.put/3","doc":"Set the value at the given index pair `{i, j}`.","ref":"CEM.MapMatrix.html#put/3"},{"type":"function","title":"CEM.MapMatrix.shape/1","doc":"Return the shape `{n_row, n_col}` of the `MapMatrix`.","ref":"CEM.MapMatrix.html#shape/1"},{"type":"function","title":"CEM.MapMatrix.to_list/1","doc":"Convert the `MapMatrix` to a list of lists of values.\n\nThe result can be passed directly to `Nx.tensor/1` to get the tensor\nrepresentation of the matrix.","ref":"CEM.MapMatrix.html#to_list/1"},{"type":"function","title":"CEM.MapMatrix.zero/1","doc":"Create a `MapMatrix` with the shape `{n_row, n_col}` and all values set to\nzero.","ref":"CEM.MapMatrix.html#zero/1"},{"type":"type","title":"CEM.MapMatrix.index/0","doc":"","ref":"CEM.MapMatrix.html#t:index/0"},{"type":"type","title":"CEM.MapMatrix.shape/0","doc":"","ref":"CEM.MapMatrix.html#t:shape/0"},{"type":"type","title":"CEM.MapMatrix.t/0","doc":"","ref":"CEM.MapMatrix.html#t:t/0"},{"type":"module","title":"CEM.Problem","doc":"Module defining a `Problem` struct.","ref":"CEM.Problem.html"},{"type":"function","title":"CEM.Problem.new/1","doc":"","ref":"CEM.Problem.html#new/1"},{"type":"function","title":"CEM.Problem.replace/3","doc":"","ref":"CEM.Problem.html#replace/3"},{"type":"function","title":"CEM.Problem.schema/0","doc":"","ref":"CEM.Problem.html#schema/0"},{"type":"type","title":"CEM.Problem.draw/0","doc":"Function to draw a random instance from the probability distribution with the given\nparameters.","ref":"CEM.Problem.html#t:draw/0"},{"type":"type","title":"CEM.Problem.init/0","doc":"Function to initialize the parameters of the probability distribution used to\ngenerate candidate solutions.","ref":"CEM.Problem.html#t:init/0"},{"type":"type","title":"CEM.Problem.instance/0","doc":"Canidate solution.","ref":"CEM.Problem.html#t:instance/0"},{"type":"type","title":"CEM.Problem.opts/0","doc":"Map of options used internally.","ref":"CEM.Problem.html#t:opts/0"},{"type":"type","title":"CEM.Problem.params/0","doc":"Parameters of the the probability distribution used to generate candidate\nsolutions.","ref":"CEM.Problem.html#t:params/0"},{"type":"type","title":"CEM.Problem.score/0","doc":"Function to score a candidate solution.","ref":"CEM.Problem.html#t:score/0"},{"type":"type","title":"CEM.Problem.score_value/0","doc":"Score value of a candidate solution.","ref":"CEM.Problem.html#t:score_value/0"},{"type":"type","title":"CEM.Problem.smooth/0","doc":"Function to smooth the parameters of the probability distribution by linearly\ninterpolating between the most recent sample-based update and the parameters\nfrom the previous optimization iteration.","ref":"CEM.Problem.html#t:smooth/0"},{"type":"type","title":"CEM.Problem.step/0","doc":"Iteration count for the CEM search.","ref":"CEM.Problem.html#t:step/0"},{"type":"type","title":"CEM.Problem.t/0","doc":"","ref":"CEM.Problem.html#t:t/0"},{"type":"type","title":"CEM.Problem.terminate?/0","doc":"Function to decide if the search should be terminated based on the log so far.\nIf not provided, the search will end when the `n_step_max` is reached (see docs\nfor `CEM.search/2`).","ref":"CEM.Problem.html#t:terminate?/0"},{"type":"type","title":"CEM.Problem.update/0","doc":"Function to update the parameters of the probability distribution using a\nsample of candidate solutions.","ref":"CEM.Problem.html#t:update/0"},{"type":"module","title":"CEM.Random","doc":"Convenience functions for generating random variates.","ref":"CEM.Random.html"},{"type":"function","title":"CEM.Random.bernoulli/1","doc":"Generate a bernoulli variate (0 or 1) with probability `p` of being 1.","ref":"CEM.Random.html#bernoulli/1"},{"type":"function","title":"CEM.Random.choice/1","doc":"Generate a random value from a list of `{value, weight}` tuples.\n\nThe probability of selecting a value is proportional to its weight.\nThe weights do not need to be normalized.\n\nAn exception is raised if the list is empty.","ref":"CEM.Random.html#choice/1"},{"type":"function","title":"CEM.Random.normal/2","doc":"Generate a normal variate with mean `mean` and standard deviation `std`.","ref":"CEM.Random.html#normal/2"},{"type":"function","title":"CEM.Random.uniform/0","doc":"Generate a uniform variate from 0 to 1.","ref":"CEM.Random.html#uniform/0"},{"type":"function","title":"CEM.Random.uniform/2","doc":"Generate a uniform variate from `min_value` to `max_value`.","ref":"CEM.Random.html#uniform/2"},{"type":"function","title":"CEM.Random.uniform_int/2","doc":"Generate a random integer from `min_value` to `max_value` (inclusive).","ref":"CEM.Random.html#uniform_int/2"},{"type":"module","title":"CEM.RankHeap","doc":"Module that defines a `RankHeap` as an adaptation of `PairingHeap`.\n\nThe rank heap maintains a maximum number of key-value pairs with the highest\nor lowest keys in a stream.","ref":"CEM.RankHeap.html"},{"type":"function","title":"CEM.RankHeap.new/2","doc":"Create a new rank heap.","ref":"CEM.RankHeap.html#new/2"},{"type":"function","title":"CEM.RankHeap.root_key/1","doc":"Return the root key of the rank heap.","ref":"CEM.RankHeap.html#root_key/1"},{"type":"function","title":"CEM.RankHeap.update/3","doc":"Update the rank heap with a new key-value pair.","ref":"CEM.RankHeap.html#update/3"},{"type":"function","title":"CEM.RankHeap.values/1","doc":"Return the values in the rank heap.","ref":"CEM.RankHeap.html#values/1"},{"type":"type","title":"CEM.RankHeap.key/0","doc":"","ref":"CEM.RankHeap.html#t:key/0"},{"type":"type","title":"CEM.RankHeap.mode/0","doc":"","ref":"CEM.RankHeap.html#t:mode/0"},{"type":"type","title":"CEM.RankHeap.t/0","doc":"","ref":"CEM.RankHeap.html#t:t/0"},{"type":"type","title":"CEM.RankHeap.value/0","doc":"","ref":"CEM.RankHeap.html#t:value/0"},{"type":"module","title":"CEM.Sample","doc":"Module for generating a sample of instance-score pairs from the\ninstance-generating probability distribution.","ref":"CEM.Sample.html"},{"type":"function","title":"CEM.Sample.generate_elite_sample_and_score/4","doc":"Given parameters of the instance-generating probability distribution,\ngenerate the elite sample of instances and the smallest (maximization\nproblem) or largest (minimization problem) score of that sample.","ref":"CEM.Sample.html#generate_elite_sample_and_score/4"},{"type":"function","title":"CEM.Sample.reduce_to_elite_sample_and_score/2","doc":"Reduce a stream of instance-score pairs into the elite sample and the score\nassociated with that sample.","ref":"CEM.Sample.html#reduce_to_elite_sample_and_score/2"},{"type":"function","title":"CEM.Sample.stream_instances_and_scores/3","doc":"Return a stream of instance-score pairs.","ref":"CEM.Sample.html#stream_instances_and_scores/3"},{"type":"type","title":"CEM.Sample.draw_fn/0","doc":"","ref":"CEM.Sample.html#t:draw_fn/0"},{"type":"type","title":"CEM.Sample.instance/0","doc":"","ref":"CEM.Sample.html#t:instance/0"},{"type":"type","title":"CEM.Sample.opts/0","doc":"","ref":"CEM.Sample.html#t:opts/0"},{"type":"type","title":"CEM.Sample.params/0","doc":"","ref":"CEM.Sample.html#t:params/0"},{"type":"type","title":"CEM.Sample.score/0","doc":"","ref":"CEM.Sample.html#t:score/0"},{"type":"type","title":"CEM.Sample.score_fn/0","doc":"","ref":"CEM.Sample.html#t:score_fn/0"},{"type":"type","title":"CEM.Sample.step/0","doc":"","ref":"CEM.Sample.html#t:step/0"},{"type":"module","title":"CEM.SearchValidators","doc":"Validation for `CEM.search/2` options.\n\nThese functions are here so that custom validations can be added as\npublic functions.","ref":"CEM.SearchValidators.html"},{"type":"function","title":"CEM.SearchValidators.validate_range/3","doc":"Returns `{:ok, x}` if `low <= x <= high`, and `{:error, message}` otherwise.","ref":"CEM.SearchValidators.html#validate_range/3"},{"type":"module","title":"CEM.Stats","doc":"Helper functions for common statistical aggregates.","ref":"CEM.Stats.html"},{"type":"function","title":"CEM.Stats.sample_mean/2","doc":"Compute the mean of a sample of numbers.\n\nIf the sample list is empty, `nil` is returned.","ref":"CEM.Stats.html#sample_mean/2"},{"type":"function","title":"CEM.Stats.sample_mean_and_std/1","doc":"Compute the mean and standard deviation of a sample of numbers.\n\nIf the sample list is empty, `{nil, nil}` is returned.","ref":"CEM.Stats.html#sample_mean_and_std/1"},{"type":"function","title":"CEM.Stats.sample_std/3","doc":"Compute the standard deviation of a sample of numbers.\n\nIf the sample list is empty, `nil` is returned.","ref":"CEM.Stats.html#sample_std/3"},{"type":"module","title":"CEM.Update","doc":"Update and smooth parameters of the instance-generating probability distribution.","ref":"CEM.Update.html"},{"type":"function","title":"CEM.Update.update_and_smooth/5","doc":"Update the parameters from a sample and then smooth based on the previous parameter values.","ref":"CEM.Update.html#update_and_smooth/5"},{"type":"type","title":"CEM.Update.instance/0","doc":"","ref":"CEM.Update.html#t:instance/0"},{"type":"type","title":"CEM.Update.opts/0","doc":"","ref":"CEM.Update.html#t:opts/0"},{"type":"type","title":"CEM.Update.params/0","doc":"","ref":"CEM.Update.html#t:params/0"},{"type":"type","title":"CEM.Update.smooth_fn/0","doc":"","ref":"CEM.Update.html#t:smooth_fn/0"},{"type":"type","title":"CEM.Update.update_fn/0","doc":"","ref":"CEM.Update.html#t:update_fn/0"},{"type":"extras","title":"README","doc":"_Note: This library is still in the early phases of development._\n\n`CEM` is an Elixir framework for applying the \n[cross-entropy method](https://en.wikipedia.org/wiki/Cross-entropy_method) to \ncontinuous and discrete optimization problems.\n\nThis library caters specifically to problems where the probability distribution\nfor generating candidate solutions belongs to the \n[exponential family](https://en.wikipedia.org/wiki/Exponential_family). This is\nnot a very restrictive limitation, since a wide range of problems can be solved\nusing either normal or Bernoulli random variables. \n\nA [separate repo](https://github.com/epfahl/cem_examples) contains livebooks \nthat demonstrate usage examples and setups for certain classes of optimization \nproblems.","ref":"readme.html"}],"content_type":"text/markdown"}